/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.gui;


import java.awt.*;


/**
* This class provides static utility methods that deal with <code>Component</code>s.
* <p>
* Like typical java gui code, this class is not multithread safe:
* it expects to only be called by {@link EventQueue}'s {@link EventQueue#isDispatchThread dispatch thread}.
* <p>
* @author Brent Boyer
*/
public class ComponentUtil {


	/** This private constructor suppresses the default (public) constructor, ensuring non-instantiability. */
    private ComponentUtil() {}


	/**
	* Returns component itself if it is a Frame,
	* or the parent Frame which contains component if it is inside a Frame,
	* or null if it is not inside a Frame.
	* <p>
	* @throws IllegalArgumentException if component == null
	*/
	public static Frame getParentFrame(Component component) throws IllegalArgumentException {
		// component checked by getParentWindow below
		
		Window parentWindow = getParentWindow(component);
		return (parentWindow instanceof Frame) ? (Frame) parentWindow : null;
	}


	/**
	* Returns component itself if it is a Window,
	* or the parent Window which contains component if it is inside a Window,
	* or null if it is not inside a Window.
	* <p>
	* @throws IllegalArgumentException if component == null
	*/
	public static Window getParentWindow(Component component) throws IllegalArgumentException {
		if (component == null) throw new IllegalArgumentException("component == null");
		
		for (; component != null; component = component.getParent()) {
			if (component instanceof Window) return (Window) component;
		}
		return null;
	}


}
