/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.gui;


import java.awt.*;


/**
* This class provides static utility methods that deal with Containers.
* <p>
* Like typical java gui code, this class is not multithread safe:
* it expects to only be called by {@link EventQueue}'s {@link EventQueue#isDispatchThread dispatch thread}.
* <p>
* @author Brent Boyer
*/
public class ContainerUtil {


	/** This private constructor suppresses the default (public) constructor, ensuring non-instantiability. */
    private ContainerUtil() {}


	/**
	* Returns the width which is available for child Components (i.e. takes container's Insets into account).
	* <p>
	* @throws IllegalArgumentException if container == null
	* @see javax.swing.JFrame
	*/
	public static int getAvailableWidth(Container container) {
		if (container == null) throw new IllegalArgumentException("container == null");

		Insets insets = container.getInsets();
		return container.getSize().width - (insets.left + insets.right);
	}


	/**
	* Returns the height which is available for child Components (i.e. takes container's Insets into account).
	* <p>
	* @throws IllegalArgumentException if container == null
	* @see javax.swing.JFrame
	*/
	public static int getAvailableHeight(Container container) {
		if (container == null) throw new IllegalArgumentException("container == null");

		Insets insets = container.getInsets();
		return container.getSize().height - (insets.top + insets.bottom);
	}


}
