/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.gui;


import java.awt.*;
import java.awt.event.*;


/**
* This class provides static utility methods that deal with MouseEvents.
* <p>
* This class is multithread safe: it is stateless.
* <p>
* @author Brent Boyer
*/
public class MouseUtil {


	/** This private constructor suppresses the default (public) constructor, ensuring non-instantiability. */
	private MouseUtil() {}


	/** Determines whether the <code>MouseEvent</code> represents a click of the leftmost mouse button (buttion #1). */
	public static boolean isLeftMouseButtonClick(MouseEvent me) {
		return ((me.getModifiers() & InputEvent.BUTTON1_MASK) != 0);
	}


	/** Determines whether the <code>MouseEvent</code> represents a click of the middle mouse button (buttion #2). */
	public static boolean isMiddleMouseButtonClick(MouseEvent me) {
		return ((me.getModifiers() & InputEvent.BUTTON2_MASK) != 0);
	}


	/** Determines whether the <code>MouseEvent</code> represents a click of the rightmost mouse button (buttion #3). */
	public static boolean isRightMouseButtonClick(MouseEvent me) {
		return ((me.getModifiers() & InputEvent.BUTTON3_MASK) != 0);
	}


	/** Determines whether the <code>MouseEvent</code> represents a single click. */
	public static boolean isSingleClick(MouseEvent me) {
		return (me.getClickCount() == 1);
	}


	/** Determines whether the <code>MouseEvent</code> represents a double click. */
	public static boolean isDoubleClick(MouseEvent me) {
		return (me.getClickCount() == 2);
	}
	
	
}