/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.gui;


import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;

// SOLELY FOR THE BENEFIT OF THE TEST CLASS:
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
* This subclass of <code>Canvas</code> simply draws a rectangular shape.
* <p>
* Its width, height, and foreground color are all accessible and settable via the existing methods of Component.
* <p>
* Like typical java gui code, this class is not multithread safe:
* it expects to only be called by {@link EventQueue}'s {@link EventQueue#isDispatchThread dispatch thread}.
* <p>
* @author Brent Boyer
*/
public class RectangleCanvas extends Canvas {


	// -------------------- constants --------------------


	private static final long serialVersionUID = 1;


	// -------------------- constructor --------------------


	/**
	* Constructor.
	* Simply passes each arg to the corresponding set method.
	* <p>
	* @throws IllegalArgumentException if any of the set methods objects
	*/
	public RectangleCanvas(int width, int height, Color foreground) throws IllegalArgumentException {
		setSize(width, height);
		setForeground(foreground);
	}


	// -------------------- paint --------------------


	public void paint(Graphics g) {
		g.fillRect(0, 0, getWidth(), getHeight());
	}


	// -------------------- UnitTest (static inner class) --------------------


	/** See the Overview page of the project's javadocs for a general description of this unit test class. */
	public static class UnitTest {

		public static void main(String[] args) throws Exception {
				// see http://java.sun.com/docs/books/tutorial/uiswing/misc/threads.html for why must do ALL swing gui in event dispatch thread (the earlier advice given in http://java.sun.com/developer/JDCTechTips/2003/tt1208.html#1 to only do the displaying in the event dispatch thread is actually wrong)
			EventQueue.invokeLater( new Runnable() {
				public void run() {
					JFrame jframe = buildFrame("RectangleCanvas.UnitTest");
					jframe.setVisible(true);
				}
			} );
		}

		protected static JFrame buildFrame(String title) {
			JFrame jframe = new JFrame(title);
			jframe.setLocation(0, 0);
			jframe.setSize(300, 300);
			jframe.getContentPane().add( buildContents() );
			jframe.addWindowListener( new WindowAdapter() {
				public void windowClosing(WindowEvent e) { System.exit(0); }
			} );
			return jframe;
		}

		protected static JComponent buildContents() {
			JPanel jpanel = new JPanel();
			jpanel.setLayout( new LineLayout(BoxLayout.Y_AXIS) );
			jpanel.setAlignmentX(0.5f);
			jpanel.add( new RectangleCanvas(50, 10, Color.RED) );
			return jpanel;
		}

	}


}
