/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.io.filefilter;


/**
* File filter that accepts files generated by a digital camera.
* <p>
* Currently, this file filter accepts only those files that end with the following file extensions (case irrelevant):
* <ol>
*  <li><code>.avi</code></li>
*  <li><code>.jpg</code> or <code>.jpeg</code></li>
*  <li><code>.thm</code></li>
* </ol>
* <p>
* For discussion of the .avi & .thm files produced by Canon digital cameras see
* <ol>
*  <li><a href="http://photography-on-the.net/forum/viewtopic.php?TopicID=6334">http://photography-on-the.net/forum/viewtopic.php?TopicID=6334</a></li>
*  <li><a href="http://www.extensis.com/portfolio/31j.html#raw11">http://www.extensis.com/portfolio/31j.html#raw11</a></li>
* </ol>
* Summary: .thm files are .jpg files (with some optional extra textual information too?) meant for thumbnails.
* <p>
* Note that this class is both a subclass of {@link javax.swing.filechooser.FileFilter javax.swing.filechooser.FileFilter}
* (so that it can be passed to {@link javax.swing.JFileChooser}),
* as well as an implementation of {@link java.io.FileFilter java.io.FileFilter}
* (so that it is suitable as an argument to {@link java.io.File#listFiles(java.io.FileFilter) File.listFiles}).
* <p>
* This class is multithread safe: it is immutable (both its immediate state, as well as the deep state of its fields).
* <p>
* @author Brent Boyer
*/
public class CameraFileFilter extends FilenameSuffixFilter {


	private static final String[] SUFFIXES = new String[] {".avi", ".jpg", ".jpeg", ".thm"};
	private static final String DESCRIPTION = FilenameSuffixFilter.makeDescription("Digital Camera Files", SUFFIXES);


	/** Constructs a new CameraFileFilter instance; directories are accepted. */
	public CameraFileFilter() {
		this(true);
	}


	/** Constructs a new CameraFileFilter instance with the specification for whether to accept directories or not. */
	public CameraFileFilter(boolean acceptDirectory) {
		super(SUFFIXES, false, (acceptDirectory ? ACCEPT_DIRECTORIES : REJECT_DIRECTORIES) );
	}


	/** Returns a description of this filter. */
	public String getDescription() { return DESCRIPTION; }


}
