/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.io.filefilter;


import java.io.*;


/**
* File filter that accepts all normal files and rejects everything else (e.g. directories).
* <p>
* Note that this class is both a subclass of {@link javax.swing.filechooser.FileFilter javax.swing.filechooser.FileFilter}
* (so that it can be passed to {@link javax.swing.JFileChooser}),
* as well as an implementation of {@link java.io.FileFilter java.io.FileFilter}
* (so that it is suitable as an argument to {@link java.io.File#listFiles(java.io.FileFilter) File.listFiles}).
* <p>
* This class is multithread safe: it is immutable (both its immediate state, as well as the deep state of its fields).
* <p>
* @author Brent Boyer
*/
public class NormalFileFilter extends javax.swing.filechooser.FileFilter implements java.io.FileFilter {


	protected static final String DESCRIPTION = "Normal Files Only";


	/** Constructs a new NormalFileFilter instance. */
	public NormalFileFilter() {}


	/** Reports whether or not the supplied file is accepted by this filter. */
	public boolean accept(File file) {
		return file.isFile();
	}


	/** Returns a description of this filter. */
	public String getDescription() { return DESCRIPTION; }


}
