/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.io.filefilter;


import java.io.*;


/**
* This file filter accepts all files. One of the constructors allows you to specify whether
* or not to accept directories also. (The default is to accept directories.)
* <p>
* Note that this class is both a subclass of {@link javax.swing.filechooser.FileFilter javax.swing.filechooser.FileFilter}
* (so that it can be passed to {@link javax.swing.JFileChooser}),
* as well as an implementation of {@link java.io.FileFilter java.io.FileFilter}
* (so that it is suitable as an argument to {@link java.io.File#listFiles(java.io.FileFilter) File.listFiles}).
* <p>
* This class is multithread safe: it is immutable (both its immediate state, as well as the deep state of its fields).
* <p>
* @author Brent Boyer
*/
public class OmniFilter extends javax.swing.filechooser.FileFilter implements java.io.FileFilter {


	private final boolean acceptDirectory;


	/** Constructs a new OmniFilter instance; directories are accepted. */
	public OmniFilter() {
		this(true);
	}


	/** Constructs a new OmniFilter instance with the specification for whether to accept directories or not. */
	public OmniFilter(boolean acceptDirectory) {
		this.acceptDirectory = acceptDirectory;
	}


	/**
	* Reports whether or not the supplied File is accepted by this filter.
	* <p>
	* @throws IllegalArgumentException if arg file is null
	* @throws SecurityException if a security manager exists and its SecurityManager.checkRead(java.lang.String) method denies read access to the file
	*/
	public boolean accept(File file) {
		if (file == null) throw new IllegalArgumentException("arg file is null");

		if ( file.isDirectory() )
			return acceptDirectory;
		else
			return true;
	}


	/** Returns a description of this filter. */
	public String getDescription() { return "All Files (*.*)"; }


}
