/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.io.filefilter;


import java.io.*;

import bb.io.ZipUtil;


/**
* Filter that accepts everything which passes {@link ZipUtil#isZippable ZipUtil.isZippable}.
* <p>
* Note that this class is both a subclass of {@link javax.swing.filechooser.FileFilter javax.swing.filechooser.FileFilter}
* (so that it can be passed to {@link javax.swing.JFileChooser}),
* as well as an implementation of {@link java.io.FileFilter java.io.FileFilter}
* (so that it is suitable as an argument to {@link File#listFiles File.listFiles}).
* <p>
* This class has no immedaite mutable state.
* It is multithread safe if the type that is assigned to its {@link #listener} field is multithread safe.
* <p>
* @author Brent Boyer
*/
public class ZippableFilter extends javax.swing.filechooser.FileFilter implements java.io.FileFilter {


	protected static final String DESCRIPTION = "All which are zippable by Java";

	protected final FileFilterListener listener;


	/**
	* Constructor.
	* <p>
	* @param listener an optional {@link FileFilterListener} to receive File accept/reject events; may be null
	*/
	public ZippableFilter(FileFilterListener listener) {
		this.listener = listener;
	}


	/**
	* Reports whether or not the supplied file is accepted by this instance.
	* Before return, notifies any {@link FileFilterListener} of the event.
	*/
	public boolean accept(File file) {
		boolean accepted = ZipUtil.isZippable(file);
		if (listener != null) {
			if (accepted) listener.accepted(file, "passed ZipUtil.isZippable");
			else listener.rejected(file, "FAILED ZipUtil.isZippable");
		}
		return accepted;
	}


	/** Returns a description of this filter. */
	public String getDescription() { return DESCRIPTION; }


}
