/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


package bb.util;


import java.io.Serializable;
import java.util.Comparator;
import java.util.SortedSet;
import java.util.TreeSet;

import org.junit.Assert;
import org.junit.Test;


/**
* Comparator implementation that ranks two Strings irrespective of their case.
* The current implementation does this by converting both to lower case
* before calling the usual {@link String#compareTo String.compareTo} method.
* <p>
* This class is multithread safe: it is stateless.
* <p>
* @author Brent Boyer
*/
public class CaseInsensitiveComparator implements Comparator<String>, Serializable {
	
	
	// -------------------- constants --------------------
	
	
	private static final long serialVersionUID = 1;
	
	
	// -------------------- compare --------------------
	
	
	public int compare(String s1, String s2) throws IllegalArgumentException {
		if (s1 == null) throw new IllegalArgumentException("s1 == null");
		if (s2 == null) throw new IllegalArgumentException("s2 == null");
		
		return s1.toLowerCase().compareTo( s2.toLowerCase() );
	}


	// -------------------- UnitTest (static inner class) --------------------


	/** See the Overview page of the project's javadocs for a general description of this unit test class. */
	public static class UnitTest {
		
		@Test public void test_compare() {
			SortedSet<String> set = new TreeSet<String>( new CaseInsensitiveComparator() );
			set.add("aa");
			set.add("aA");
			set.add("Aa");
			set.add("AA");
			Assert.assertEquals(1, set.size());
		}
		
	}
	
	
}
