/*
Copyright  2008 Brent Boyer

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Lesser GNU General Public License for more details.

You should have received a copy of the Lesser GNU General Public License along with this program (see the license directory in this project).  If not, see <http://www.gnu.org/licenses/>.
*/


/*
+++ email junit guys:

1) Description needs more detail than getDisplayName, such as class and method
that, or at least they need to commit to what the format will be so i can parse it

2) need details about when run started and stopped for timing purposes
	--this would be in analogy to Result.getRunTime(), altho that method is poor because it does not seem to use the high resolution System.nanoTime, so change that too

3) when do javadocs, link to sun online stuff so get rid of the ugliness

4) javadoc for RunListener.testIgnored needs to mention that both @Ignore and @Test needed for notification

javadoc for RunListener.testFailure needs to mention that it is called <i>in between</i> calls to testStarted and testFinished,
in the event that the test method throws a Throwable.

5) how do you get access to @Ignore's message text?  it does not appear to be part of description.getDisplayName() or description.toString()

6) Failure.getException() returns a Throwable (as it ought to), but its method name is wrong; ought to be the more accurate getThrowable()

7) errors in their online javadocs, e.g.
	http://junit.sourceforge.net/javadoc_40/index.html
	has %nbsp; when the % should be an &

8) Description.getChildren returns a ArrayList<Description> instead of a generic List

9) strange api: Request.filterWith(...) returns a NEW Request instance that appears to be the old one with plus with filtering added
	why not simply have it be void and apply the filter to the CURRENT instance?

10) in addition to supplying a Class[], need to be able to supply a Method[] too

11) Filter.describe() is unnecessary; should be toString
	should Filter redeclare it as abstract to force subclass to override, and is this possible in Java?

12) BUG:
	--if run a Request which uses a Filter, as i do in the code below and IF there is an error in one of your Test classes
		--for example, an @Test method which is declared with args
	--then the Runner fails to be created with a stacktrace like this:
		java.lang.Exception: Method report should have no parameters
			at org.junit.internal.runners.MethodValidator.validateTestMethods(MethodValidator.java:80)
			at org.junit.internal.runners.MethodValidator.validateInstanceMethods(MethodValidator.java:28)
			at org.junit.internal.runners.MethodValidator.validateMethodsForDefaultRunner(MethodValidator.java:43)
			at org.junit.internal.runners.JUnit4ClassRunner.validate(JUnit4ClassRunner.java:36)
			at org.junit.internal.runners.JUnit4ClassRunner.<init>(JUnit4ClassRunner.java:27)
			at sun.reflect.NativeConstructorAccessorImpl.newInstance0(Native Method)
			at sun.reflect.NativeConstructorAccessorImpl.newInstance(NativeConstructorAccessorImpl.java:39)
			at sun.reflect.DelegatingConstructorAccessorImpl.newInstance(DelegatingConstructorAccessorImpl.java:27)
			at java.lang.reflect.Constructor.newInstance(Constructor.java:513)
			at org.junit.internal.requests.ClassRequest.buildRunner(ClassRequest.java:33)
			at org.junit.internal.requests.ClassRequest.getRunner(ClassRequest.java:28)
			at org.junit.internal.requests.ClassesRequest.getRunner(ClassesRequest.java:21)
			at org.junit.internal.requests.FilterRequest.getRunner(FilterRequest.java:33)
			at org.junit.runner.JUnitCore.run(JUnitCore.java:109)
			at bb.util.JUnitExecutor.run(JUnitExecutor.java:253)
			at bb.util.JUnitExecutor.main(JUnitExecutor.java:196)

	--this error will ought to always be reported to any RunListeners
	--HOWEVER, this error will first be run by the Filter, but will be given the method name "initializationError0"
	--so, this error will fail to be reported to any RunListeners if the Filter is not prepared for a method with that name
I put in a temp hack in the code below to always catch this, however, it is really a JUnit logic problem

13) Assert needs an assertFails method, as an alternative to @Test(expected=Exception.class), so that can have multiple failures per test method (the tests may use common resources that are annoying to have to put into fields)

14) very bad choice with Assert to put error message as first argument

15) Assert.assertArrayEquals(double[], double[]) and Assert.assertArrayEquals(double[], double[], double epsilon) need to be supplied




+++ investigate better sounds

+++ is there any use for the suite functionality for me?

+++ todo in future:
	--write a GuiNotifier which could be used instead of ConsoleNotifier, and could do things a lot prettier
	--if do this, require the user to specify all the Notifiers that he wants attached as args to main
*/


package bb.util;


import java.awt.Toolkit;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.io.Serializable;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.junit.Test;
import org.junit.runner.*;
import org.junit.runner.manipulation.Filter;
import org.junit.runner.notification.*;

import bb.io.ConsoleUtil;
import bb.io.FileUtil;
import bb.io.DirUtil;
import bb.io.StreamUtil;
import bb.io.filefilter.ClassFilter;
import bb.science.FormatUtil;


/**
* Discovers and autoexecutes all the <a href="http://junit.sourceforge.net">JUnit 4</a> test methods
* (i.e. those annotated <code>{@link Test}</code>) found in the class files of some directory tree.
* Offers filtering to fine tune which packages, classes, and methods
* in that directory tree are actually run.
* <p>
* While executing test methods, a {@link ConsoleNotifier} instance
* and a {@link AudioNotifier} are used to provide feedback.
* <p>
* This class is multithread safe: it is immutable (both its immediate state, as well as the deep state of its fields).
* <p>
* @author Brent Boyer
*/
public class JUnitExecutor {
	
	
	// -------------------- switch constants --------------------
	
	
	private static final String root_key = "-root";
	private static final String packages_key = "-packages";
	private static final String classes_key = "-classes";
	private static final String methods_key = "-methods";
	
	/** Specifies all the switch keys which can legally appear as command line arguments to {@link #main main}. */
	private static final Set<String> JUnitExecutor_keys = new HashSet<String>( Arrays.asList(
		root_key, packages_key, classes_key, methods_key
	) );
	
	
	// -------------------- main and helper methods --------------------
	
	
	/**
	* Creates a new <code>JUnitExecutor</code> instance and calls its <code>{@link #run run}</code> method.
	* <p>
	* The <code>root</code> param in that call to <code>run</code>
	* is specified as a key/value pair in <code>args</code> of the form
	* <code>-root <i>pathToRootDirectory</i></code>.
	* <p>
	* The <code>pcmFilter</code> param in that call to <code>run</code>
	* is specified using any combination of these filters:
	* <ol>
	*  <li>
	*		a key/value pair in <code>args</code> of the form
	*		<code>-packages <i>listOfPackageNames</i></code>,
	*		for example <code>-packages bb.io,bb.util</code> or <code>-packages bb.*</code>,
	*		to restrict the running of test methods to only classes whose packages are in the list
	*  </li>
	*  <li>
	* 		a key/value pair in <code>args</code> of the form
	*		<code>-classes <i>listOfSimpleClassNames</i></code>,
	*		for example <code>-classes Benchmark,Bootstrap</code> or
	*		<code>-classes ArrayUtil\$UnitTest,Benchmark\$UnitTest</code> or
	*		<code>-classes B.*</code>,
	*		to restrict the running of test methods to only classes whose <i>simple</i> names (no package part)
	*		are in the list
	*  </li>
	*  <li>
	* 		a key/value pair in <code>args</code> of the form
	*		<code>-methods <i>listOfMethodNames</i></code>,
	*		for example <code>-classes test_concatenate,test_shuffle</code> or <code>-classes te.*</code>,
	*		to restrict the running of test methods to only those whose names
	*		are in the list
	*  </li>
	* </ol>
	* A given test method is only run if it passes every filter (i.e. there is an implied AND of all the filters).
	* <p>
	* For each filter, the value must be a single comma separated list with no spaces in the list.
	* The list is parsed into elements, <i>each of which is interpreted as a
	* <a href="http://java.sun.com/javase/6/docs/api/java/util/regex/Pattern.html#sum">java regular expression</a></i>.
	* So, wildcards can be used as in the examples above.
	* A given test method is run if it matches at least one element in the list (i.e. there is an implied OR of all the list elements).
	* <p>
	* @throws Exception of some kind if any is raised
	*/
	public static void main(String[] args) throws Exception {
		Properties2 switches = CommandLineUtil.parseArgs(args);
		CommandLineUtil.checkSwitchKeys(switches, JUnitExecutor_keys, "for " + JUnitExecutor.class.getName());
		
		String root = switches.getMandatoryProperty(root_key);
		Set<Pattern> packageReqs = parseList(packages_key, switches);
		Set<Pattern> classReqs = parseList(classes_key, switches);
		Set<Pattern> methodReqs = parseList(methods_key, switches);
		PcmFilter pcmFilter = new PcmFilter(packageReqs, classReqs, methodReqs);
		
		JUnitExecutor junitExecutor = new JUnitExecutor();
		junitExecutor.run(root, pcmFilter);
	}
	
	
	private static Set<Pattern> parseList(String key, Properties2 switches) throws Exception {
		Set<Pattern> set = null;
		if (switches.containsKey(key)) {
			set = new LinkedHashSet<Pattern>();	// use LinkedHashSet to guarantee that the iteration order is the same order as what the user specified on the command line
			String[] requirements = switches.getProperty(key).split(",", -1);
			for (String regex : requirements) {
				Pattern pattern = Pattern.compile(regex);
				set.add(pattern);
			}
		}
		return set;
	}
	
	
	// -------------------- constructor --------------------
	
	
	/** Constructor. */
	public JUnitExecutor() {}
	
	
	// -------------------- run and helper methods --------------------
	
	
	/**
	* Determines all the classes underneath <code>root</code> which contain JUnit 4 test methods
	* (i.e. those which are annotated <code>@{@link Test}</code>)
	* that pass <code>pcmFilter</code>.
	* Then runs those tests using <code>JUnit 4</code>.
	* <p>
	* <i>Note that <code>root</code> must be the sole directory which is the package root;
	* it cannot be just any directory which contains the classes.</i>
	* For example, suppose that the directory <code>D:/software/java/projectsMine/bbLibrary/class</code>
	* is the package root which contains class files like <code>A.class</code> and <code>B.class</code>
	* (these would be in the default, unnamed package)
	* as well as subdirectories that contain more class files
	* (these would be in named packages).
	* Then, <code>root</code> must have the value <code>D:/software/java/projectsMine/bbLibrary/class</code>.
	* You would not use higher level directories like
	* <code>D:/software/java/projectsMine/bbLibrary</code> or <code>D:/software/java/projectsMine</code>
	* in this case because they are not the package root.
	* <p>
	* @throws Exception of some kind if any is raised
	*/
	public void run(String root, PcmFilter pcmFilter) throws Exception {
		JUnitCore jUnitCore = new JUnitCore();
		jUnitCore.addListener( new ConsoleNotifier() );
		jUnitCore.addListener( new AudioNotifier() );
		
		Class[] classesWithTests = findClassesWithJUnitTests(root, pcmFilter);
		Request request = Request.classes("classes with Tests in root = " + root, classesWithTests);
		request = request.filterWith(pcmFilter);	// Note that pcmFilter is used both here, to cause the JUnit framework to reject methods that should not be run, as well as in the call to findClassesWithJUnitTests above
		
		jUnitCore.run(request);
	}
	
	
	private Class[] findClassesWithJUnitTests(String root, PcmFilter pcmFilter) throws Exception {
		Set<Class> result = new LinkedHashSet<Class>();
		Set<Class> classesNeedTests = new LinkedHashSet<Class>();
		SortedSet<String> problems = new TreeSet<String>();	// use SortedSet because loadClass will start each entry with the class file's path, so it would be nice if they were listed in order
		String pathRoot = getPath( new File(root) );
		File[] classFiles = DirUtil.getFilesInTree( new File(pathRoot), new ClassFilter() );
		try {
			for (File classFile : classFiles) {
				ConsoleUtil.overwriteLine("findClassesWithJUnitTests: " + classFile.getPath());
				
				String packageName = extractPackageName(classFile, pathRoot);
				String className = extractClassName(classFile);
				if (!pcmFilter.shouldRun(packageName, className, null)) continue;
				
				Class c = loadClass(classFile, pathRoot, problems);
				if (c == null) continue;
				
				if (containsJUnitTests(c, pcmFilter)) {
					boolean added = result.add(c);
					if (!added) throw new IllegalStateException("failed to add a Class when expected to");
				}
				else if (needsJUnitTests(c)) {
					classesNeedTests.add(c);	// only add if is a top level class
				}
			}
		}
		finally {
			ConsoleUtil.eraseLine();
		}
		if (problems.size() > 0) {
			System.err.println();
			System.err.println("The following " + problems.size() + " problems were encountered that caused the corresponding class files to be skipped:");
			System.err.println( "\t" + StringUtil.toString(problems, "\n\t") );
		}
		if (classesNeedTests.size() > 0) {
			System.out.println();
			System.out.println("The following " + classesNeedTests.size() + " top level classes lack an inner class which has JUnit test methods:");
			System.out.println( "\t" + StringUtil.toString(classesNeedTests, "\n\t") );
		}
		return result.toArray( new Class[result.size()] );
	}
	
	
	/** Returns a standard form for file's path: the separator is always '/' and directories end with a '/'. */
	private String getPath(File file) {
		String path = file.getPath().replace('\\', '/');
		if (file.isDirectory() && !path.endsWith("/")) path += "/";
		return path;
	}
	
	
	/** Note: returns an empty String if classFile is in the default (unnamed) package. */
	private String extractPackageName(File classFile, String pathRoot) throws IllegalStateException {
		String path = getPath(classFile);
		if (!path.startsWith(pathRoot)) throw new IllegalStateException("path = " + path + " !startsWith pathRoot = " + pathRoot);
		path = path.substring( pathRoot.length() );
		
		int i = path.lastIndexOf('/');
		if (i == -1) return "";
		else if (i == 0) throw new IllegalStateException("classFile = " + classFile.getPath() + " has a path after pathRoot = " + pathRoot + " has been removed which starts with '/' which is illegal");
		else if (i > 0) return path.substring(0, i).replace('/', '.');
		else throw new IllegalStateException("i = " + i + "; happened for classFile = " + classFile.getPath() + " and pathRoot = " + pathRoot);
	}
	
	
	private String extractClassName(File classFile) {
		return FileUtil.getNameMinusExtension(classFile);
	}
	
	
	private Class loadClass(File classFile, String pathRoot, SortedSet<String> problems) throws Exception {
		String path = getPath(classFile);
		if (!path.startsWith(pathRoot)) throw new IllegalStateException("path = " + path + " !startsWith pathRoot = " + pathRoot);
		if (!path.endsWith(".class")) throw new IllegalStateException("path = " + path + " !endsWith .class");
		String classNameFull = path.replace(pathRoot, "").replace(".class", "").replace('/', '.');
		try {
			StdStreams.swap();	// CRITICAL: must call this because have seen class loading cause Loggers to be initialized, and if those Loggers use a ConsoleHandler then they must be initialized with the standard streams swapped to what they will be when ConsoleNotifier is used below
// +++ maybe should only call the above if a ConsoleNotifier is being used, since only then does the output get intermixed which is bad; in other cases, the user may be expecting to see console output
			return Class.forName(classNameFull);
		}
		catch (Throwable t) {
			problems.add( classFile.getPath() + ": " + ThrowableUtil.getTypeAndMessage(t) );
			return null;
		}
		finally {
			StdStreams.unswap(null, null);
		}
	}
	
	
	private boolean containsJUnitTests(Class c, PcmFilter pcmFilter) {
		for (Method method : c.getMethods()) {
			if (!pcmFilter.shouldRun(null, null, method.getName())) continue;
			
			for (Annotation annotation : method.getAnnotations()) {
				if (annotation instanceof Test) return true;
			}
		}
		return false;
	}
	
	
	private boolean needsJUnitTests(Class c) {
return false;	// the code below works for MY needs (how I use the UnitTest inner class approach), but will not work for people who make separate top level test classes, plus it produces too much output to regularly use...
/*
		if (c.isInterface()) return false;	// interfaces obviously never need test methods
		if (c.getName().contains("$")) return false;	// inner classes are never considered to need test methods
		for (Class inner : c.getDeclaredClasses()) {
			for (Method method : inner.getMethods()) {
				for (Annotation annotation : method.getAnnotations()) {
					if (annotation instanceof Test) return false;
				}
			}
		}
		return true;
*/
	}
	
	
	// -------------------- ConsoleNotifier (static inner class) --------------------
	
	
	/**
	* Prints notification of test events to the console.
	* <p>
	* The functionality that it offers is:
	* <ol>
	*  <li>
	*		while executing test methods, intercepts the standard streams (out and err) and captures any output.
	*		The last line of output is continously overwritten to the console,
	*		to let the user know what the test method is generating.
	*		The complete output is saved to a dedicated file.
	*		This prevents it from intermixing with the console output produced by this class,
	*		makes it extremely easy to find the output from a given test method,
	*		and allows that output to be saved if necessary.
	*  </li>
	*  <li>after each test method has finished, its name, outcome, and how long it took is printed to the console</li>
	*  <li>after each run is completed, total statistics are printed to the console</li>
	* </ol>
	* <p>
	* Typical console output produced by this class (once a run has finished) looks like:
	* <pre><code>
		JUnit 4 run started for: All
		--------------------------------------------------
		somepackage.SomeClass$UnitTest
				test_method1: PASSED; execution time = 1 ms
				test_method2: IGNORED; execution time = 2 us
				test_method3: FAILED: java.lang.RuntimeException: unexpected error happened; execution time = 3 ms
		--------------------------------------------------
		anotherpackage.AnotherClass$UnitTest
				test_methodA: PASSED; execution time = 4 ms
				test_methodB: IGNORED; execution time = 5 us
				test_methodC: FAILED: java.lang.RuntimeException: unexpected error happened; execution time = 6 ms
		--------------------------------------------------
		JUnit 4 run FAILED: 2/4 tests failed (2 tests were ignored); total execution time = 100 ms
		
		Note: there are 4 test output files inside ..\log\testOutput</code></pre>
	* <p>
	* <b>Warning:</b> this class terminates the JVM if any unexpected Throwable is encountered.
	* (This does not include any Throwable associated with a test {@link Failure}, as that is expected.)
	* <p>
	* This class is multithread safe: every public method is synchronized.
	*/
	private static class ConsoleNotifier extends RunListener {
		
		private static final File testOutputDirectory = new File("../log/testOutput");
		static {
			DirUtil.ensureExists(testOutputDirectory);
		}
		
		private long timeRunStartNs;	// keep track ourselves because Result.getRunTime() does not use the high resolution System.nanoTime
		private long timeTestStartNs;
		
		private String className;
		private String methodName;
		
		File fileOut;
		File fileErr;
		
		private boolean ignored;
		private Failure failure;
		
		private final SortedMap<String,SortedSet<String>> classToMethodsFailed = new TreeMap<String,SortedSet<String>>();
		
		private ConsoleNotifier() {}
		
		// -------------------- RunListener methods --------------------
		
		public synchronized void testRunStarted(Description description) {
			try {
				System.out.println();
				System.out.println( "JUnit 4 run started for: " + description.getDisplayName() );
				className = null;
				methodName = null;
				fileOut = null;
				fileErr = null;
				timeRunStartNs = System.nanoTime();
			}
			catch (Throwable t) {
				onProblem("An unexpected Throwable was caught", t);
			}
		}
		
		public synchronized void testStarted(Description description) {
			try {
				boolean sameClass = parseNames(description);
				if (!sameClass) {
					System.out.println("--------------------------------------------------");
					System.out.println(className);
				}
				System.out.print("\t" + methodName + ": ");
				fileOut = new File(testOutputDirectory, className + "." + methodName + ".stdOut.txt");
				fileErr = new File(testOutputDirectory, className + "." + methodName + ".stdErr.txt");
				StdStreams.swap();	// CRITICAL: after call this method, no point in the code below should use the std streams until testFinished/onProblem calls restoreStdStreams
				ignored = false;
				failure = null;
				timeTestStartNs = System.nanoTime();
			}
			catch (Throwable t) {
				onProblem("An unexpected Throwable was caught", t);
			}
		}
		
		/**
		* Note: the Junit 4 javadocs do not describe this method well (as of 2008/5/8),
		* but it currently is called <i>instead of</i> {@link #testStarted testStarted}
		* if the method is annotated with both @Ignore and @Test.
		* In this case, <i>there will be no subsequent call</i> to {@link #testFinished testFinished};
		* only this method is called.
		*/
		public synchronized void testIgnored(Description description) {
			try {
				testStarted(description);
// +++ need to find a way to get any message text of the @Ignore
				ignored = true;
				testFinished(description);
			}
			catch (Throwable t) {
				onProblem("An unexpected Throwable was caught", t);
			}
		}
		
		/**
		* Called <i>in between</i> calls to {@link #testStarted testStarted}
		* and {@link #testFinished testFinished} if a test method fails.
		*/
		public synchronized void testFailure(Failure failure) {
			try {
				this.failure = failure;
				failure.getException().printStackTrace(System.err);	// Note that System.err currently points to stdErrBuffer, i.e. the log file, at this point
				
				SortedSet<String> methodsFailed = classToMethodsFailed.get(className);
				if (methodsFailed == null) {
					methodsFailed = new TreeSet<String>();
					classToMethodsFailed.put(className, methodsFailed);
				}
				methodsFailed.add(methodName);
			}
			catch (Throwable t) {
				onProblem("An unexpected Throwable was caught", t);
			}
		}
		
		public synchronized void testFinished(Description description) {
			try {
				long timeTestStopNs = System.nanoTime();
				
				StdStreams.unswap(fileOut, fileErr);
				
				if (!ignored && (failure == null)) System.out.print("\t" + methodName + ": PASSED");
				else if (!ignored && (failure != null)) System.out.print("\t" + methodName + ": FAILED: " + ThrowableUtil.getTypeAndMessage(failure.getException()) + "; full stack trace in log file" );
				else if (ignored && (failure == null)) System.out.print("\t" + methodName + ": IGNORED");
				else throw new IllegalStateException("the combination ignored = " + ignored + " && failure = " + failure + " should never happen");
				
				double timeExecS = (timeTestStopNs - timeTestStartNs) * 1e-9;	// e-9 converts ns to s as required below
				System.out.println( "; execution time = " + FormatUtil.toEngineeringTime(timeExecS) );
			}
			catch (Throwable t) {
				onProblem("An unexpected Throwable was caught", t);
			}
		}
		
		public synchronized void testRunFinished(Result result) {
			try {
				long timeRunStopNs = System.nanoTime();
				double timeRunExecS = (timeRunStopNs - timeRunStartNs) * 1e-9;	// e-9 converts ns to s as needed by FormatUtil.toEngineeringTime below
				String execTime = "; total execution time = " + FormatUtil.toEngineeringTime(timeRunExecS);
				
				System.out.println("--------------------------------------------------");
				
				if (result.wasSuccessful()) {
					System.out.println("JUnit 4 run was successful: all " + result.getRunCount() + " tests passed (" + result.getIgnoreCount() + " tests were ignored)" + execTime);
				}
				else {
					System.out.println("JUnit 4 run FAILED: " + result.getFailureCount() + "/" + result.getRunCount() + " tests failed (" + result.getIgnoreCount() + " tests were ignored)" + execTime);
					System.out.println();
					System.out.println("Summary of each class which failed, followed by a list of that class's failing methods:");
					System.out.println( "\t" + StringUtil.toString(classToMethodsFailed, "\n\t") );
				}
				
				if (testOutputDirectory.exists()) {
					int numberOutputFiles = DirUtil.getContents(testOutputDirectory).length;
					if (numberOutputFiles > 0) {
						System.out.println();
						System.out.println( "Note: there are " + numberOutputFiles + " test output files inside " + testOutputDirectory.getPath() );
					}
				}
			}
			catch (Throwable t) {
				onProblem("An unexpected Throwable was caught", t);
			}
		}
		
		// -------------------- helper methods --------------------
		
		/**
		* Handles problems generated by the JUnit framework itself (including this class),
		* as opposed to Throwables raised by the test methods
		* (which should get caught by the JUnit framework and passed to {@link #testFailure testFailure}).
		* Because any such problem represents some serious issue with the framework,
		* it is handled by printing to the console followed by JVM termination.
		*/
		private void onProblem(String message, Throwable t) {
			try {
				try {
					StdStreams.unswap(fileOut, fileErr);	// CRITICAL: must call this first before do any output below, since have no idea what state are in when called
				}
				catch (Throwable t2) {
					System.err.println();
					t2.printStackTrace(System.err);
				}
				
				System.err.println();
				if (!StringUtil.isBlank(message)) System.err.println(message);
				if (t != null) t.printStackTrace(System.err);
			}
			finally {
				System.err.println("PROGRAM WILL NOW ABORT");
				System.exit(1);
			}
		}
		
		/**
		* Parses the class name and method name out of description
		* and assigns the results to {@link #className} and {@link #methodName}.
		* <p>
		* @return true if className is the same as before, false otherwise
		*/
		private boolean parseNames(Description description) throws IllegalStateException {
			String classNamePrevious = className;
			DescriptionFields descriptionFields = new DescriptionFields(description);
			className = descriptionFields.getClassNameFull();
			methodName = descriptionFields.methodName;
			return className.equals(classNamePrevious);
		}
		
	}
	
	
	// -------------------- StdStreams (static inner class) --------------------
	
	
	/**
	* Swaps and unswaps the standard streams (System.out and System.err)
	* for a pair of {@link BufferWithEcho} instances.
	* <p>
	* This class guarantees that the same replacement streams are always used
	* across all calls to {@link #swap swap} and {@link #unswap unswap}.
	* In the context of the containing class, this behavior is required in order to coordinate
	* the use of this class by {@link #loadClass loadClass} and {@link ConsoleNotifier}. 
	* This class satisfies this guarantee by being entirely static based,
	* since that is an easy way to assure singleton behavior.
	* <p>
	* This class is multithread safe: every method is synchronized.
	*/
	private static class StdStreams {
		
		private static final PrintStream stdOutOriginal = System.out;
		private static final PrintStream stdErrOriginal = System.err;
		private static final BufferWithEcho stdOutBuffer = new BufferWithEcho(System.out);
		private static final BufferWithEcho stdErrBuffer = new BufferWithEcho(System.err);

		private static synchronized void swap() {
			if (System.out != stdOutOriginal) throw new IllegalStateException("System.out != stdOutOriginal");
			if (System.err != stdErrOriginal) throw new IllegalStateException("System.err != stdErrOriginal");
			
			stdOutBuffer.reset();	// CRITICAL: must do this before reuse it
			System.setOut( new PrintStream(stdOutBuffer, true) );
			
			stdErrBuffer.reset();	// CRITICAL: must do this before reuse it
			System.setErr( new PrintStream(stdErrBuffer, true) );
		}
		
		/**
		* First unswaps the standard streams to their original objects,
		* then writes any contents of the {@link BufferWithEcho} instances to either the corresponding File args
		* (fileOut and fileErr), if non-null, or else to the original standard stream.
		* <p>
		* @throws IllegalStateException if System.out == stdOutOriginal; System.err == stdErrOriginal
		* @throws Exception if one or more Throwables are raised during execution; the message will contain the stack traces of each one
		*/
		private static synchronized void unswap(File fileOut, File fileErr) throws Exception {
			if (System.out == stdOutOriginal) throw new IllegalStateException("System.out == stdOutOriginal");
			if (System.err == stdErrOriginal) throw new IllegalStateException("System.err == stdErrOriginal");
			
			StringBuilder sb = new StringBuilder();
			
			try {
				System.out.flush();	// CRITICAL: only flush, do NOT close, since may be used again
				System.setOut(stdOutOriginal);
			}
			catch (Throwable t) {
				sb.append( ThrowableUtil.toString(t) );
			}
			
			try {
				System.err.flush();	// CRITICAL: only flush, do NOT close, since may be used again
				System.setErr(stdErrOriginal);
			}
			catch (Throwable t) {
				sb.append( ThrowableUtil.toString(t) );
			}
			
			try {
				if (stdOutBuffer.size() > 0) {
					stdOutBuffer.eraseLine();	// clear any output that was on the ORIGINAL std out from the calls to BufferWithEcho.echoLastLine (which are implicitly done by every call to its write method)
					if (fileOut != null) {
						FileUtil.writeBytes( stdOutBuffer.toByteArray(), fileOut, false );
					}
					else {
						stdOutOriginal.print( new String( stdOutBuffer.toByteArray() ) );
					}
				}
			}
			catch (Throwable t) {
				sb.append( ThrowableUtil.toString(t) );
			}
			
			try {
				if (stdErrBuffer.size() > 0) {
					stdErrBuffer.eraseLine();	// clear any output that was on the ORIGINAL std err from the calls to BufferWithEcho.echoLastLine (which are implicitly done by every call to its write method)
					if (fileErr != null) {
						FileUtil.writeBytes( stdErrBuffer.toByteArray(), fileErr, false );
					}
					else {
						stdErrOriginal.print( new String( stdErrBuffer.toByteArray() ) );
					}
				}
			}
			catch (Throwable t) {
				sb.append( ThrowableUtil.toString(t) );
			}
			
			if (sb.length() > 0) throw new Exception("The following problem(s) occured:" + sb.toString());
		}
		
	}
	
	
	// -------------------- BufferWithEcho (static inner class) --------------------
	
	
	/**
	* Modifies superclass to echo the last line stored in the buffer to the console
	* after byte(s) are written to it.
	* This gives the user some amount of feedback as to what is going on.
	* <p>
	* The console output is continuously overwritten on the same line in order to reduce clutter,
	* just like the {@link ConsoleUtil} class does.
	* <p>
	* <i>It is critical that this stream be closed when its use is over,</i>
	* since the {@link #close close} method has been overridden to erase the console line
	* that it has been writing to.
	* <p>
	* This class is multithread safe: every public method is synchronized.
	*/
	private static class BufferWithEcho extends ByteArrayOutputStream {
		
		private final PrintStream stdStreamOriginal;
		
		private BufferWithEcho(PrintStream stdStreamOriginal) {
			this.stdStreamOriginal = stdStreamOriginal;
		}
		
		@Override public synchronized void write(int b) {
			super.write(b);
			echoLastLine();
		}
		
		@Override public synchronized void write(byte[] b, int off, int len) {
			super.write(b, off, len);
			echoLastLine();
		}
		
		private void echoLastLine() {
			assert (count > 0) : "echoLastLine called when count = " + " is not > 0";
			
			int end = count - 1;	// end is the ending index (non-inclusive) of the last line in buf, and does not include the end of line char(s)
			for (; end >= 0; end--) {
				if (!CharUtil.isLineEnd(buf[end])) break;
			}
			++end;	// restore end to the previous line end index (which may be the implicit one just beyond the last valid index of buf)
			
			int start = end - 1;	// start is the starting index (inclusive) of the last line in buf
			for (; start >= 0; start--) {
				if (CharUtil.isLineEnd(buf[start])) break;
			}
			++start;	// restore start to the previous NON line end index
			
			byte[] bytes = Arrays.copyOfRange(buf, start, end);
			writeLine( new String(bytes) );
		}
		
		private void eraseLine() {
			writeLine("");
		}
		
		private void writeLine(String line) {
			stdStreamOriginal.print( ConsoleUtil.clearLine );
			stdStreamOriginal.print( StringUtil.keepWithinLength(line, ConsoleUtil.maxCharsPerLine) );
			stdStreamOriginal.flush();
		}
		
		/** Overrides superclass method to additionally erase the console line that this instance has been writing to. */
		@Override public synchronized void close() throws IOException {
			super.close();
			eraseLine();
		}
		
	}
	
	
	// -------------------- AudioNotifier (static inner class) --------------------
	
	/**
	* Provides audio feedback for test events.
	* <p>
	* This class is multithread safe: every public method is synchronized.
	*/
	private static class AudioNotifier extends RunListener {
		
		private AudioNotifier() {}
		
		// -------------------- RunListener methods --------------------
		
		public synchronized void testRunStarted(Description description) {
		}
		
		public synchronized void testStarted(Description description) {
//Toolkit.getDefaultToolkit().beep();
		}
		
		/**
		* Note: the Junit 4 javadocs do not describe this method well (as of 2008/5/8),
		* but it currently is called <i>instead of</i> {@link #testStarted testStarted}
		* if the method is annotated with both @Ignore and @Test.
		* In this case, <i>there will be no subsequent call</i> to {@link #testFinished testFinished};
		* only this method is called.
		*/
		public synchronized void testIgnored(Description description) {
		}
		
		/**
		* Note: the Junit 4 javadocs do not describe this method well (as of 2008/5/8),
		* but it currently is called <i>in between</i> calls to {@link #testStarted testStarted}
		* and {@link #testFinished testFinished} in the event that the test method throws a Throwable.
		*/
		public synchronized void testFailure(Failure failure) {
Toolkit.getDefaultToolkit().beep();
// +++ use the better stuff from Sounds class if can figure out how to put it from eg into bblib and reference the wav files somehow?
		}
		
		public synchronized void testFinished(Description description) {
		}
		
		public synchronized void testRunFinished(Result result) {
			if (result.wasSuccessful()) {
Toolkit.getDefaultToolkit().beep();
// +++ use the better stuff from Sounds class if can figure out how to put it from eg into bblib and reference the wav files somehow?
			}
			else {
Toolkit.getDefaultToolkit().beep();
Toolkit.getDefaultToolkit().beep();
// +++ use the better stuff from my Sounds class if can figure out how to put the .wav files into a jar file and reference them
			}
		}
		
	}
	
	
	// -------------------- PcmFilter (static inner class) --------------------
	
	
	/**
	* Filters on the basis of <i>p</i>ackage, <i>c</i>lass, and <i>m</i>ethod names (pcm).
	* <p>
	* This class is multithread safe: every public method is synchronized.
	*/
	private static class PcmFilter extends Filter implements Serializable {
		
		private static final long serialVersionUID = 1;
		
		/** @serial */
		private final Set<Pattern> packageReqs;
		
		/** @serial */
		private final Set<Pattern> classReqs;
		
		/** @serial */
		private final Set<Pattern> methodReqs;
		
		private PcmFilter(Set<Pattern> packageReqs, Set<Pattern> classReqs, Set<Pattern> methodReqs) {
			this.packageReqs = packageReqs;
			this.classReqs = classReqs;
			this.methodReqs = methodReqs;
		}
		
		public synchronized String describe() { return toString(); }
		
		@Override public synchronized String toString() {
			return
				"Accepts every package whose name matches all these regular expressions: " + StringUtil.toString(packageReqs, ", ")
				+ "; accepts every class whose (simple) name matches all these regular expressions: " + StringUtil.toString(classReqs, ", ")
				+ "; accepts every method whose name matches all these regular expressions: " + StringUtil.toString(methodReqs, ", ");
		}
		
		public synchronized boolean shouldRun(Description description) {
			return shouldRun( new DescriptionFields(description) );
		}
		
		public synchronized boolean shouldRun(DescriptionFields descriptionFields) {
			return shouldRun( descriptionFields.packageName, descriptionFields.className, descriptionFields.methodName );
		}
		
		private boolean shouldRun(String packageName, String className, String methodName) {
if ((methodName != null) && methodName.equals("initializationError0")) return true;
// +++ above is a hack to deal with a JUnit bug (in my opinion--see notes at top of class)
			return
				matches(packageName, packageReqs) &&
				matches(className, classReqs) &&
				matches(methodName, methodReqs);
		}
		
		private boolean matches(String s, Set<Pattern> requirements) {
			if (s == null) return true;
			if (requirements == null) return true;
			
			for (Pattern p : requirements) {
				if (p.matcher(s).matches()) return true;
			}
			
			return false;
		}
		
	}
	
	
	// -------------------- DescriptionFields (static inner class) --------------------
	
	
	/**
	* Parses and stores fields from a {@link Description}.
	* This class was only introduced because Description fails to offer the necessary API.
	* <p>
	* This class is multithread safe: it is immutable (both its immediate state, as well as the deep state of its fields).
	*/
	private static class DescriptionFields {
		
		private final String packageName;
		private final String className;
		private final String methodName;
		
		private DescriptionFields(Description description) {
			String s = description.getDisplayName();
			String pkgClName;
			if (s.contains("(")) {
				if (!s.contains(")")) throw new IllegalStateException( "description.getDisplayName() = " + s + " contains'(' but does not contain ')'" );
				String[] tokens = s.split("\\(|\\)", 0);	// matches the chars '(' or ')'; CRITICAL: must use 0 to drop the final empty token that is otherwise produced (e.g. if use -1)
				if (tokens.length != 2) throw new IllegalStateException( "tokens.length = " + tokens.length + " != 2; happened for description.getDisplayName() = " + s + " which produced tokens = " + StringUtil.toString(tokens, ",") );
				pkgClName = tokens[1];
				methodName = tokens[0];
			}
			else {
				pkgClName = s;
				methodName = null;
			}
			int i = pkgClName.lastIndexOf('.');
			if (i == -1) {
				packageName = null;
				className = pkgClName;
			}
			else {
				packageName = pkgClName.substring(0, i);
				className = pkgClName.substring(i + 1);
			}
		}
		
		private String getClassNameFull() {
			if (packageName == null) return className;
			return packageName + "." + className;
		}
		
	}
	
	
	// -------------------- UnitTest (static inner class) --------------------
	
	
	// None needed--is tested by main
	
	
}
